// polycube:  A polyomino and polycube puzzle solver.
// Copyright 2011-2018 Matthew T. Busche.
//
// This program is free software:  you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef FIRSTHEURISTIC_HPP
#define FIRSTHEURISTIC_HPP

#include "OrderingHeuristic.hpp"
#include "fastbits.hpp"

class FirstHeuristic : public OrderingHeuristic
{
    public:
        double gridPointValue(const GridPoint* gp) const override;
        ImageListSet selectFila(int nrp, int nrs, const gridbitfield_t occupancy, bool dlxPristine) const;
        virtual std::ostream& print(std::ostream& os) const override;
};

inline double FirstHeuristic::gridPointValue(const GridPoint* gp) const
{
    return gp->id;
}

ImageListSet FirstHeuristic::selectFila(int nrp, int nrs, const gridbitfield_t occupancy, bool dlxPristine) const
{
    return entity->getImageListSet(entity->filaGrid[FastBits::lowestSetBit(~occupancy)], occupancy);
}

inline std::ostream& FirstHeuristic::print(std::ostream& os) const
{
    return os << "f";
}


#endif
