// polycube:  A polyomino and polycube puzzle solver.
// Copyright 2011-2018 Matthew T. Busche.
//
// This program is free software:  you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef LINEARHEURISTIC_HPP
#define LINEARHEURISTIC_HPP

#include "OrderingHeuristic.hpp"

class LinearHeuristic : public OrderingHeuristic
{
    private:
        double a;
        double b;
        double c;

    public:
        LinearHeuristic(double a, double b, double c);
        double gridPointValue(const GridPoint* gp) const;
        virtual std::ostream& print(std::ostream& os) const override;
};

inline LinearHeuristic::LinearHeuristic(double a, double b, double c)
    :
        a(a), b(b), c(c)
{
}

inline double LinearHeuristic::gridPointValue(const GridPoint* gp) const
{
    // Adding a small linear factor of the gridpoint id as a klunky way of
    // breaking ties.  Not really important, I just like a predictable result
    // from the sort/prioritization.
    //
    return a*gp->getX() + b*gp->getY() + c*gp->getZ() + gp->id / 1e7;
}

std::ostream& LinearHeuristic::print(std::ostream& os) const
{
    return os << "l@" << a << "," << b << "," << c;
}

#endif
