// polycube:  A polyomino and polycube puzzle solver.
// Copyright 2011-2018 Matthew T. Busche.
//
// This program is free software:  you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "OrderingHeuristic.hpp"
#include "OrderingHeuristicStore.hpp"
#include <algorithm>

void OrderingHeuristic::initSelectionOrder(
        int numFilaGridPoints,
        GridPoint const * const * filaGrid,
        int* selectionOrder) const
{
    double value[maxFilaGridPoints];
    for(int g = 0; g < numFilaGridPoints; ++g)
        value[g] = gridPointValue(filaGrid[g]);
    std::sort(
            selectionOrder,
            selectionOrder+numFilaGridPoints,
            [&value](int i1, int i2) { return value[i1] < value[i2]; });
}


DlxHead* OrderingHeuristic::selectDlx() const
{
    DlxHead* best = NULL;
    double bestValue = 1e30;
    for(DlxHead* head = (DlxHead*) entity->dlxRoot->right;
        head->isGridPoint(); head = (DlxHead*) head->right)
    {
        // Overriding normal behavior in favor of columns with only 1 or 0 fits.
        //
        if(head->numRow <= 1)
        {
            best = head;
            for( ; head->isGridPoint(); head = (DlxHead*) head->right)
                if(head->numRow == 0)
                    return head;
            return best;
        }
        double value = gridPointValue((GridPoint*) head);
        if(value < bestValue)
        {
            bestValue = value;
            best = head;
        }
    }
    return best;
}


ImageListSet OrderingHeuristic::selectFila(int nrp, int nrs, const gridbitfield_t occupancy, bool dlxPristine) const
{
    int g = entity->lastSelectionIndex[nrp+1] + 1;
    while(occupancy & entity->filaGrid[entity->selectionOrder[g]]->bit)
        ++g;
    entity->lastSelectionIndex[nrp] = g;
    return entity->getImageListSet(entity->filaGrid[entity->selectionOrder[g]], occupancy);
}

double OrderingHeuristic::gridPointValue(const GridPoint* gp) const
{
    std::ostringstream errMsg;
    errMsg << "OrderingHeuristic " << *this << " does not implement gridPointValue().";
    throw std::runtime_error(errMsg.str());
}
