// polycube:  A polyomino and polycube puzzle solver.
// Copyright 2011-2018 Matthew T. Busche.
//
// This program is free software:  you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef RADIALHEURISTIC_HPP
#define RADIALHEURISTIC_HPP

#include "OrderingHeuristic.hpp"

class RadialHeuristic : public OrderingHeuristic
{
    private:
        double xc;
        double yc;
        double zc;

    public:
        RadialHeuristic(double xc, double yc, double zc);
        double gridPointValue(const GridPoint* gp) const override;
        virtual std::ostream& print(std::ostream& os) const override;
};

inline RadialHeuristic::RadialHeuristic(double xc, double yc, double zc)
    :
        xc(xc), yc(yc), zc(zc)
{
}

inline double RadialHeuristic::gridPointValue(const GridPoint* gp) const
{
    double dx = gp->getX() - xc;
    double dy = gp->getY() - yc;
    double dz = gp->getZ() - zc;

    // Adding a small linear factor of the gridpoint id as a klunky way of
    // breaking ties.  Not really important, I just like a predictable result
    // from the sort/prioritization.
    //
    return -dx*dx - dy*dy - dz*dz + gp->id / 1e7;
}

inline std::ostream& RadialHeuristic::print(std::ostream& os) const
{
    return os << "R@" << xc << "," << yc << "," << zc;
}

#endif
